from django.apps import AppConfig
from django.core.checks import Error, register


class MailerIntegrationConfig(AppConfig):
    default_auto_field = "django.db.models.BigAutoField"
    name = "basxconnect.mailer_integration"

    def ready(self):
        import basxconnect.mailer_integration.signal_handlers  # noqa


@register()
def check_mailchimp_settings(app_configs, **kwargs):
    required_settings = [
        "MAILCHIMP_API_KEY",
        "MAILCHIMP_SERVER",
        "MAILCHIMP_LIST_ID",
        "MAILCHIMP_SEGMENT_ID",
        "MAILCHIMP_INTERESTS_CATEGORY_ID",
    ]
    errors = []
    for setting in required_settings:
        errors.extend(_check_setting(setting))

    return errors


def _check_setting(required_setting):
    from django.conf import settings

    if not hasattr(settings, required_setting):
        error = Error(
            f"setting.{required_setting} is not defined",
            id=f"basxconnect.mailer_integration.ERROR{required_setting}",
        )
        return [error]
    else:
        return []



# Generated by Django 3.2.9 on 2021-12-16 09:46

from django.db import migrations, models


class Migration(migrations.Migration):
    def init_default_sorting_name(apps, _):
        naturalpersons = apps.get_model("core.NaturalPerson").objects.all()
        other_persons = (
            *apps.get_model("core.LegalPerson").objects.all(),
            *apps.get_model("core.PersonAssociation").objects.all(),
        )

        for person in naturalpersons:
            person.default_sorting_name = person.last_name
            person.save()

        for person in other_persons:
            person.default_sorting_name = person.name
            person.save()

    dependencies = [
        ("core", "0025_alter_term_slug"),
    ]

    operations = [
        migrations.AddField(
            model_name="historicallegalperson",
            name="default_sorting_name",
            field=models.CharField(
                blank=True, max_length=255, verbose_name="Default Sorting Name"
            ),
        ),
        migrations.AddField(
            model_name="historicalnaturalperson",
            name="default_sorting_name",
            field=models.CharField(
                blank=True, max_length=255, verbose_name="Default Sorting Name"
            ),
        ),
        migrations.AddField(
            model_name="historicalperson",
            name="default_sorting_name",
            field=models.CharField(
                blank=True, max_length=255, verbose_name="Default Sorting Name"
            ),
        ),
        migrations.AddField(
            model_name="historicalpersonassociation",
            name="default_sorting_name",
            field=models.CharField(
                blank=True, max_length=255, verbose_name="Default Sorting Name"
            ),
        ),
        migrations.AddField(
            model_name="person",
            name="default_sorting_name",
            field=models.CharField(
                blank=True, max_length=255, verbose_name="Default Sorting Name"
            ),
        ),
        migrations.RunPython(init_default_sorting_name, migrations.RunPython.noop),
    ]



import hashlib
from typing import List

import mailchimp_marketing
from django.conf import settings
from mailchimp_marketing.api_client import ApiClientError

from basxconnect.mailer_integration.abstract import mailer
from basxconnect.mailer_integration.abstract.mailer import MailerPerson, MailingInterest
from basxconnect.mailer_integration.mailchimp.parsing import (
    create_mailer_person_from_raw,
)
from basxconnect.mailer_integration.models import Interest


class Mailchimp(mailer.AbstractMailer):
    def __init__(self) -> None:
        self.client = mailchimp_marketing.Client()
        self.client.set_config(
            {"api_key": settings.MAILCHIMP_API_KEY, "server": settings.MAILCHIMP_SERVER}
        )

    def name(self) -> str:
        return "Mailchimp"

    def get_person_count(self) -> int:
        return self.client.lists.get_segment(
            list_id=settings.MAILCHIMP_LIST_ID,
            segment_id=settings.MAILCHIMP_SEGMENT_ID,
        )["member_count"]

    def get_persons(self, count: int, offset: int) -> List[MailerPerson]:
        segment = self.client.lists.get_segment_members_list(
            list_id=settings.MAILCHIMP_LIST_ID,
            segment_id=settings.MAILCHIMP_SEGMENT_ID,
            count=count,
            offset=offset,
            include_cleaned=True,
            include_transactional=True,
            include_unsubscribed=True,
            # TODO: Find out why this does not work. Would be nice to request as little data as possible.
            # fields=[
            #     "email_address",
            #     "merge_fields.FNAME",
            #     "merge_fields.LNAME",
            # ],
        )
        return [
            create_mailer_person_from_raw(raw_person)
            for raw_person in segment["members"]
        ]

    def delete_person(self, email: str):
        self.client.lists.delete_list_member(
            settings.MAILCHIMP_LIST_ID, compute_email_hash(email)
        )

    def put_person(self, person: MailerPerson, **kwargs):
        email_hash = compute_email_hash(person.email)
        self.client.lists.set_list_member(
            settings.MAILCHIMP_LIST_ID,
            email_hash,
            {
                "email_address": person.email,
                "status_if_new": person.status,
                "interests": compute_interests_dict(person),
                "merge_fields": {
                    "FNAME": person.first_name or person.display_name,
                    "LNAME": person.last_name,
                    **(
                        settings.MAILCHIMP_ADDITIONAL_MERGE_FIELDS(person)
                        if hasattr(settings, "MAILCHIMP_ADDITIONAL_MERGE_FIELDS")
                        else {}
                    ),
                },
                "language": person.language,
                **kwargs,
            },
        )
        if hasattr(settings, "MAILCHIMP_TAG"):
            self.client.lists.update_list_member_tags(
                settings.MAILCHIMP_LIST_ID,
                email_hash,
                {"tags": [{"name": settings.MAILCHIMP_TAG, "status": "active"}]},
            )

    def add_person(self, person: MailerPerson):
        self.put_person(person, status=person.status)

    def email_exists(self, email: str) -> bool:
        try:
            self.client.lists.get_list_member(
                settings.MAILCHIMP_LIST_ID, compute_email_hash(email)
            )
        except ApiClientError as e:
            if e.status_code == 404:
                return False
            raise e
        return True

    def change_email_address(self, old_email: str, new_email: str):
        self.client.lists.update_list_member(
            settings.MAILCHIMP_LIST_ID,
            compute_email_hash(old_email),
            {"email_address": new_email},
        )

    def get_interests(self):
        return [
            MailingInterest(interest["id"], interest["name"])
            for interest in self.client.lists.list_interest_category_interests(
                settings.MAILCHIMP_LIST_ID,
                settings.MAILCHIMP_INTERESTS_CATEGORY_ID,
            )["interests"]
        ]

    def tag(self) -> str:
        return "Imported from Mailchimp"


def compute_interests_dict(person) -> dict:
    interests = dict(
        [(interest.external_id, False) for interest in Interest.objects.all()]
    )
    for interest_id in person.interests_ids:
        interests[interest_id] = True
    return interests


def compute_email_hash(email):
    # required by mailchimp, ignore security check
    return hashlib.md5(str(email).lower().encode()).hexdigest()  # nosec



# Generated by Django 3.2.4 on 2021-11-22 09:38

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("mailer_integration", "0014_rename_mailingpreferences_subscription"),
    ]

    operations = [
        migrations.AddField(
            model_name="synchronizationperson",
            name="message",
            field=models.CharField(blank=True, max_length=255, verbose_name="Message"),
        ),
    ]



# Generated by Django 3.2.5 on 2021-09-28 08:34

from django.db import migrations, models

import basxconnect.core.layouts.editperson.common.base_data


class Migration(migrations.Migration):
    def copy_tags(apps, scheme_editor):
        for person in apps.get_model("core.Person").objects.all():
            person.tags.set(person.categories.all())
            person.save()

    dependencies = [
        ("core", "0015_rename_category_to_tag"),
    ]

    operations = [
        migrations.AddField(
            model_name="person",
            name="tags",
            field=models.ManyToManyField(
                blank=True, limit_choices_to={"vocabulary__slug": "tag"}, to="core.Term"
            ),
        ),
        migrations.AlterField(
            model_name="person",
            name="categories",
            field=models.ManyToManyField(
                blank=True,
                limit_choices_to={"vocabulary__slug": "tag"},
                related_name="_core_person_categories_+",
                to="core.Term",
            ),
        ),
        migrations.RunPython(copy_tags, migrations.RunPython.noop),
    ]



# Generated by Django 3.2.4 on 2021-08-23 14:30

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("core", "0011_auto_20210726_2047"),
        ("mailer_integration", "0004_auto_20210823_1424"),
    ]

    operations = [
        migrations.AlterField(
            model_name="mailingpreferences",
            name="email",
            field=models.OneToOneField(
                on_delete=django.db.models.deletion.CASCADE, to="core.email"
            ),
        ),
    ]



# Generated by Django 3.1.7 on 2021-05-23 04:40

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("core", "0009_auto_20210522_1817"),
    ]

    operations = [
        migrations.AlterField(
            model_name="email",
            name="type",
            field=models.ForeignKey(
                blank=True,
                limit_choices_to={"category__slug": "emailtype"},
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="type_core_email_list",
                to="core.term",
            ),
        ),
    ]



from bread import layout as layout
from bread import menu
from bread.layout.base import DEVMODE_KEY
from bread.utils.links import Link
from bread.utils.urls import reverse, reverse_model
from django.utils.translation import gettext_lazy as _

import basxconnect.core.models

from .. import models


# New menu group with developer mode checking
class DevGroup(menu.Group):
    def has_permission(self, request):
        return super().has_permission(request) and request.session.get(
            DEVMODE_KEY, False
        )


class SuperUserItem(menu.Item):
    def has_permission(self, request):
        return super().has_permission(request) and request.user.is_superuser


R = layout.grid.Row
C = layout.grid.Col
F = layout.forms.FormField

# MENU ENTRIES ---------------------------------------------------------------------

menu.registeritem(
    menu.Item(
        Link(
            reverse("core.vocabulary.browse"),
            basxconnect.core.models.Vocabulary._meta.verbose_name_plural,
        ),
        menu.admingroup,
    )
)

persongroup = menu.Group(_("Persons"), iconname="group")

menu.registeritem(
    menu.Item(
        Link(reverse_model(models.Person, "browse"), _("Persons"), iconname="group"),
        persongroup,
    )
)

menu.registeritem(
    menu.Item(
        Link(
            reverse("basxconnect.core.views.settings_views.generalsettings"),
            _("General"),
        ),
        menu.settingsgroup,
    )
)

menu.registeritem(
    menu.Item(
        Link(
            reverse("basxconnect.core.views.settings_views.personsettings"),
            _("Persons"),
        ),
        menu.settingsgroup,
    )
)

menu.registeritem(
    menu.Item(
        Link(
            reverse("basxconnect.core.views.settings_views.relationshipssettings"),
            _("Relationships"),
        ),
        menu.settingsgroup,
    )
)



# Generated by Django 3.2.5 on 2021-07-26 13:47

from django.db import migrations, models

import basxconnect.core.models.persons


class Migration(migrations.Migration):

    dependencies = [
        ("core", "0010_auto_20210523_1140"),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicallegalperson",
            name="personnumber",
            field=models.CharField(
                db_index=True,
                default=basxconnect.core.models.persons.random_personid,
                max_length=255,
                verbose_name="Person number",
            ),
        ),
        migrations.AlterField(
            model_name="historicalnaturalperson",
            name="personnumber",
            field=models.CharField(
                db_index=True,
                default=basxconnect.core.models.persons.random_personid,
                max_length=255,
                verbose_name="Person number",
            ),
        ),
        migrations.AlterField(
            model_name="historicalperson",
            name="personnumber",
            field=models.CharField(
                db_index=True,
                default=basxconnect.core.models.persons.random_personid,
                max_length=255,
                verbose_name="Person number",
            ),
        ),
        migrations.AlterField(
            model_name="historicalpersonassociation",
            name="personnumber",
            field=models.CharField(
                db_index=True,
                default=basxconnect.core.models.persons.random_personid,
                max_length=255,
                verbose_name="Person number",
            ),
        ),
        migrations.AlterField(
            model_name="person",
            name="personnumber",
            field=models.CharField(
                default=basxconnect.core.models.persons.random_personid,
                max_length=255,
                unique=True,
                verbose_name="Person number",
            ),
        ),
    ]



# Generated by Django 3.2.4 on 2021-08-23 14:24

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("core", "0011_auto_20210726_2047"),
        ("mailer_integration", "0003_alter_interest_external_id"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="mailingpreferences",
            name="person",
        ),
        migrations.AddField(
            model_name="mailingpreferences",
            name="email",
            field=models.OneToOneField(
                default=None,
                on_delete=django.db.models.deletion.CASCADE,
                to="core.person",
            ),
            preserve_default=False,
        ),
    ]



import abc
from typing import List, NamedTuple

from basxconnect.mailer_integration.models import Subscription


class MailerPerson(NamedTuple):
    display_name: str
    email: str
    interests_ids: List[str]
    status: str
    last_name: str = ""
    first_name: str = ""
    country: str = ""  # in alpha_2 format (e.g. "CH", "DE", ...)
    postcode: str = ""
    address: str = ""
    city: str = ""
    language: str = ""

    @staticmethod
    def from_subscription(preferences: Subscription):
        person = preferences.email.person
        return MailerPerson(
            display_name=person.name,
            first_name=(
                person.naturalperson.first_name
                if hasattr(person, "naturalperson")
                else ""
            ),
            last_name=(
                person.naturalperson.last_name
                if hasattr(person, "naturalperson")
                else ""
            ),
            email=preferences.email.email,
            interests_ids=[
                interest.external_id for interest in preferences.interests.all()
            ],
            status=preferences.status,
            language=preferences.language,
        )


class MailingInterest(NamedTuple):
    id: str
    name: str


class AbstractMailer(metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def name(self) -> str:
        pass

    @abc.abstractmethod
    def get_person_count(self) -> int:
        pass

    @abc.abstractmethod
    def get_persons(
        self,
        count: int,
        offset: int,
    ) -> List[MailerPerson]:
        pass

    @abc.abstractmethod
    def put_person(self, person: MailerPerson, **kwargs):
        pass

    @abc.abstractmethod
    def add_person(self, person: MailerPerson):
        pass

    @abc.abstractmethod
    def email_exists(self, email: str) -> bool:
        pass

    @abc.abstractmethod
    def change_email_address(self, old_email: str, new_email: str):
        pass

    @abc.abstractmethod
    def get_interests(self) -> List[MailingInterest]:
        pass

    @abc.abstractmethod
    def tag(
        self,
    ) -> str:
        pass



# Generated by Django 3.2.5 on 2021-10-04 10:08

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("core", "0018_term_slug"),
    ]

    operations = [
        migrations.AlterField(
            model_name="term",
            name="slug",
            field=models.CharField(
                max_length=255, unique=True, verbose_name="Slug", blank=True
            ),
        ),
    ]



import htmlgenerator as hg
from bread import layout
from bread.forms.forms import breadmodelform_factory
from bread.utils import pretty_modelname
from bread.utils.urls import reverse_model
from bread.views import BreadView
from django import forms
from django.apps import apps
from django.contrib.auth.mixins import PermissionRequiredMixin
from django.shortcuts import redirect
from django.urls import reverse, reverse_lazy
from django.utils.translation import gettext_lazy as _
from dynamic_preferences.registries import global_preferences_registry
from formtools.wizard.views import NamedUrlSessionWizardView

from ..models import (
    Email,
    LegalPerson,
    NaturalPerson,
    Person,
    PersonAssociation,
    Postal,
    Term,
)

ADD_FORM_LAYOUTS = {
    NaturalPerson: lambda: hg.BaseElement(
        layout.grid.Row(
            layout.grid.Col(layout.forms.FormField("first_name")),
            layout.grid.Col(layout.forms.FormField("last_name")),
        ),
        layout.grid.Row(
            layout.grid.Col(layout.forms.FormField("salutation")),
            layout.grid.Col(layout.forms.FormField("gender")),
        ),
    ),
    LegalPerson: lambda: hg.DIV(
        layout.forms.FormField("name"), layout.forms.FormField("name_addition")
    ),
    PersonAssociation: hg.DIV(layout.forms.FormField("name")),
}


def ADD_ADDRESS_LAYOUT():
    return layout.grid.Grid(
        layout.grid.Row(
            layout.grid.Col(_("Address"), style="font-weight: 700; margin-bottom: 2rem")
        ),
        layout.grid.Row(
            layout.grid.Col(
                layout.forms.FormField("address", inputelement_attrs={"rows": "2"})
            )
        ),
        layout.grid.Row(
            layout.grid.Col(
                layout.forms.FormField("postcode"),
            ),
            layout.grid.Col(layout.forms.FormField("city")),
        ),
        layout.grid.Row(layout.grid.Col(layout.forms.FormField("country"))),
        gutter=False,
    )


def ADD_EMAIL_LAYOUT():
    return layout.grid.Grid(
        layout.grid.Row(
            layout.grid.Col(_("Email"), style="font-weight: 700; margin-bottom: 2rem")
        ),
        layout.grid.Row(
            layout.grid.Col(
                layout.forms.FormField("email", elementattributes={"required": False})
            ),
            layout.grid.Col(layout.forms.FormField("type")),
        ),
        gutter=False,
    )


def generate_wizard_form(formlayout):
    # needs to be rendered in view of type NamedUrlSessionWizardView in order to work correctly
    def go_back_url(context):
        url = reverse(
            context["request"].resolver_match.view_name,
            kwargs={"step": context["wizard"]["steps"].prev},
        )
        return f"document.location='{url}'"

    return layout.forms.Form(
        hg.C("wizard.form"),
        layout.forms.Form(
            hg.C("wizard.management_form"),
            layout.forms.FormField("current_step"),
            standalone=False,
        ),
        formlayout,
        hg.DIV(
            hg.DIV(
                hg.If(
                    hg.C("wizard.steps.prev"),
                    layout.button.Button(
                        _("Back"),
                        buttontype="secondary",
                        onclick=hg.F(go_back_url),
                    ),
                ),
                hg.If(
                    hg.F(
                        lambda c: c["wizard"]["steps"].last
                        == c["wizard"]["steps"].current
                    ),
                    layout.button.Button(
                        _("Complete"), type="submit", style="margin-left: 1rem"
                    ),
                    layout.button.Button(
                        _("Continue"), type="submit", style="margin-left: 1rem"
                    ),
                ),
            ),
            style="align-items: flex-end",
            _class="bx--form-item",
        ),
    )


class SearchForm(forms.Form):
    name_of_existing_person = forms.CharField(
        label=_("Check for existing people before continuing"),
        max_length=255,
        required=False,
    )

    searchelement = layout.search.Search(
        model=Person,
        widgetattributes={
            "placeholder": _("Start typing to search for a person..."),
            "hx_get": reverse_lazy(
                "basxconnect.core.views.person.search_person_view.searchperson"
            ),
            "hx_trigger": "changed, keyup changed delay:100ms",
            "hx_target": "#search-results",
            "name": "q",
        },
    )
    # clear search field when search box is emptied
    searchelement[0][3].attributes["onclick"] = "$('#search-results').innerHTML = ''"

    title = _("Search person")
    _layout = hg.DIV(
        hg.DIV(
            _(
                "Before a new person can be added it must be confirmed that the person does not exists yet."
            ),
            style="margin-bottom: 2rem",
        ),
        searchelement,
        hg.DIV(id="search-results", style="margin-bottom: 2rem;"),
    )


class ChooseType(forms.Form):
    PERSON_TYPES = {
        "core.NaturalPerson": pretty_modelname(NaturalPerson),
        "core.LegalPerson": pretty_modelname(LegalPerson),
        "core.PersonAssociation": pretty_modelname(PersonAssociation),
    }
    persontype = forms.TypedChoiceField(
        label=_("Type of person"),
        choices=tuple(PERSON_TYPES.items()),
        coerce=lambda a: apps.get_model(a),
        empty_value=None,
    )

    title = _("Choose person main type")
    _layout = hg.BaseElement(
        hg.DIV(
            _("Please choose what type of person you want to add:"),
            style="margin-bottom: 2rem",
        ),
        layout.forms.FormField("persontype"),
    )


class ChooseSubType(forms.Form):
    ALLOWED_SUBTYPE_CATEGORY = {
        Person: None,
        NaturalPerson: "naturaltype",
        LegalPerson: "legaltype",
        PersonAssociation: "associationtype",
    }
    subtype = forms.ModelChoiceField(
        label=_("Subtype of person"), queryset=Term.objects.all(), required=False
    )

    def __init__(self, persontype, *args, **kwargs):
        super().__init__(*args, **kwargs)
        subtype_vocabulary = ChooseSubType.ALLOWED_SUBTYPE_CATEGORY.get(persontype)
        if subtype_vocabulary is None:
            self.fields = {}
        else:
            self.fields["subtype"].queryset = Term.objects.filter(
                vocabulary__slug=subtype_vocabulary
            )
            self.fields["subtype"].initial = global_preferences_registry.manager()[
                f"persons__default_{subtype_vocabulary}"
            ]

    title = _("Choose person type")
    _layout = layout.forms.FormField("subtype")


class AddPersonInformation(forms.Form):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    title = _("Add person")
    _layout = hg.DIV("Please select a person type first")


class ConfirmNewPerson(forms.Form):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    title = _("Finish")
    _layout = hg.DIV("Please select a person type first")


def generate_add_form_for(model, request, data, files, initial=None):
    form = breadmodelform_factory(
        request=request, model=model, layout=ADD_FORM_LAYOUTS[model]()
    )(data, files, initial=initial)

    for fieldname, field in breadmodelform_factory(
        request, Postal, ADD_ADDRESS_LAYOUT()
    )().fields.items():
        form.fields[fieldname] = field

    for fieldname, field in breadmodelform_factory(
        request, Email, ADD_EMAIL_LAYOUT()
    )().fields.items():
        form.fields[fieldname] = field

    formlayout = hg.BaseElement(
        layout.grid.Grid(
            ADD_FORM_LAYOUTS[model](), style="margin-bottom: 2rem", gutter=False
        ),
        layout.grid.Grid(
            layout.grid.Row(
                layout.grid.Col(ADD_ADDRESS_LAYOUT()),
                layout.grid.Col(ADD_EMAIL_LAYOUT()),
            ),
            gutter=False,
        ),
    )
    form._layout = formlayout
    return form


# The WizardView contains mostly control-flow logic and some configuration
class AddPersonWizard(PermissionRequiredMixin, BreadView, NamedUrlSessionWizardView):
    kwargs = {"url_name": "core:person:add_wizard", "urlparams": {"step": "str"}}
    urlparams = (("step", str),)
    permission_required = "core.add_person"

    form_list = [
        ("Search", SearchForm),
        ("Type", ChooseType),
        ("Subtype", ChooseSubType),
        ("Information", AddPersonInformation),
        ("Confirmation", ConfirmNewPerson),
    ]
    # translation detection
    _("Search")
    _("Type")
    _("Subtype")
    _("Information")
    _("Confirmation")
    condition_dict = {
        "Subtype": lambda wizard: ChooseSubType.ALLOWED_SUBTYPE_CATEGORY.get(
            (wizard.get_cleaned_data_for_step("Type") or {}).get("persontype")
        )
    }

    def get_person_type(self):
        return (self.get_cleaned_data_for_step("Type") or {}).get("persontype")

    def get_steps(self):
        steps = []

        for i, step in enumerate(self.get_form_list().keys()):
            status = "incomplete"
            if i < self.steps.index:
                status = "complete"
            if step == self.steps.current:
                status = "current"
            steps.append((_(step), status))
        return steps

    def get_layout(self):
        return hg.BaseElement(
            hg.H3(_("Add new person")),
            hg.H4(self.get_form().title),
            layout.progress_indicator.ProgressIndicator(
                self.get_steps(),
                style="margin-bottom: 2rem",
            ),
            layout.grid.Grid(
                layout.grid.Row(
                    layout.grid.Col(
                        generate_wizard_form(self.get_form()._layout), width=8
                    )
                ),
                gutter=False,
            ),
            hg.SCRIPT(
                hg.mark_safe(
                    r"""
                    addressTextarea = document.getElementById("id_address");
                    submitBtn = document.querySelector(".bx--content [type=\"submit\"]");
                    var addressFocused = false;
                    if (addressTextarea !== null) {
                        addressTextarea.addEventListener("focus", e => {
                            addressFocused = true;
                        });
                        addressTextarea.addEventListener("blur", e => {
                            addressFocused = false;
                        });
                    }
                    document.addEventListener("keydown", e => {
                        if (e.code === "Enter" && !addressFocused)
                            submitBtn.click();
                    });
                    """
                )
            ),
        )

    def get_form_kwargs(self, step):
        ret = super().get_form_kwargs()
        if step == "Subtype":
            ret.update({"persontype": self.get_person_type()})
        return ret

    def get_form(self, step=None, data=None, files=None):
        form = super().get_form(step, data, files)
        if step is None:
            step = self.steps.current

        # step 3 and 4 depend on the select type, so we generate the forms dynamically
        if step in ("Information", "Confirmation"):
            persontype = self.get_person_type()
            if persontype:
                if step == "Information":
                    form = generate_add_form_for(
                        persontype,
                        self.request,
                        data,
                        files,
                    )
                else:
                    form = generate_add_form_for(
                        persontype,
                        self.request,
                        data,
                        files,
                        self.get_cleaned_data_for_step("Information"),
                    )
                    form._layout.insert(
                        0,
                        layout.notification.InlineNotification(
                            "",
                            _("Review and confirm the entered information"),
                            lowcontrast=True,
                            hideclosebutton=True,
                        ),
                    )
                    for fieldname in form.fields:
                        form.fields[fieldname].disabled = True
                        form.fields[fieldname].widget.attrs["style"] = "color: #000"
                form.title = _("Add %s") % pretty_modelname(persontype)
        return form

    def done(self, form_list, **kwargs):
        personform = list(form_list)[-1]
        # in case the new person had a subtype set, we need to set the attribute here
        subtype = (self.get_cleaned_data_for_step("Subtype") or {}).get("subtype")
        if subtype:
            personform.instance.type = subtype
        newperson = personform.save()
        newperson.core_postal_list.create(
            **{
                k: v
                for k, v in personform.cleaned_data.items()
                if k in ("address", "city", "postcode", "country")
            }
        )
        if "email" in personform.cleaned_data:
            newperson.core_email_list.create(
                email=personform.cleaned_data["email"],
                type=personform.cleaned_data["type"],
            )

        newperson.save()
        return redirect(
            reverse_model(
                newperson._meta.model,
                "read",
                kwargs={"pk": newperson.pk},
            )
        )



# Generated by Django 3.2.5 on 2021-09-28 04:58

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("core", "0011_auto_20210726_2047"),
    ]

    operations = [
        migrations.AddField(
            model_name="historicalnaturalperson",
            name="autogenerate_displayname",
            field=models.BooleanField(
                default=True, verbose_name="Autogenerate Display Name"
            ),
        ),
        migrations.AddField(
            model_name="naturalperson",
            name="autogenerate_displayname",
            field=models.BooleanField(
                default=True, verbose_name="Autogenerate Display Name"
            ),
        ),
    ]



import bread.layout
import htmlgenerator as hg
from django.utils.translation import gettext_lazy as _


def sync_help_modal():
    return bread.layout.modal.Modal(
        _("Help"),
        _(
            "The button below is currently the only way of getting new subcribers from the mailer into our system. Is it also the only way of getting updates for subscribers that we already have in our system. This is what happens when the button is pressed:"
        ),
        hg.DIV(
            hg.UL(
                hg.LI(
                    _(
                        "For all the Subscriptions that are in the relevant segment (e.g. 'UM Switzerland') in the Mailer, we check whether the email address is already in BasxConnect."
                    ),
                    _class="bx--list__item",
                ),
                hg.LI(
                    _(
                        "If an email address is already in BasxConnect, the downloaded subscription will be attached to the email address and override the current values in case there are any."
                    ),
                    _class="bx--list__item",
                ),
                hg.LI(
                    _(
                        "If an email address is not yet in BasxConnect, a new person will be created with that email address."
                    ),
                    _class="bx--list__item",
                ),
                _class="bx--list--unordered",
            ),
            style="padding-left: 1rem;",
        ),
        width=8,
    )



import htmlgenerator as hg
from bread import layout
from bread.layout import ObjectFieldLabel
from bread.layout.components.icon import Icon
from django.utils.translation import gettext_lazy as _

from basxconnect.core import models
from basxconnect.core.layouts.editperson.common import addresses, base_data, utils
from basxconnect.core.layouts.editperson.common.utils import (
    display_label_and_value,
    person_metadata,
)

R = layout.grid.Row
C = layout.grid.Col


def base_data_tab(request):

    return layout.tabs.Tab(
        _("Personal data"),
        utils.grid_inside_tab(
            R(
                personal_data(),
                person_metadata(),
            ),
            base_data.common_tiles(request),
        ),
    )


def personal_data():
    displayed_fields = [
        utils.display_field_label_and_value(field)
        for field in [
            "salutation",
            "title",
            "name",
            "first_name",
            "last_name",
            "date_of_birth",
            "profession",
        ]
    ] + [
        hg.If(
            hg.C("object.deceased"),
            display_label_and_value(
                ObjectFieldLabel("deceased"),
                hg.If(hg.C("object.deceased"), _("Yes"), _("No")),
            ),
        ),
        hg.If(
            hg.C("object.deceased"),
            utils.display_field_label_and_value("decease_date"),
        ),
    ]
    return utils.tile_col_edit_modal_displayed_fields(
        _("Personal Data"),
        models.NaturalPerson,
        "ajax_edit_personal_data",
        Icon("user--profile"),
        displayed_fields,
    )


def contact_details_naturalperson(request):
    return hg.BaseElement(
        R(
            addresses.postals(),
            addresses.numbers(request),
        ),
        R(
            addresses.email(request),
            addresses.urls(request),
        ),
        R(
            base_data.tags(),
            base_data.other(),
        ),
    )



# Generated by Django 3.2.4 on 2021-11-19 08:30

import django.db.models.deletion
import django.db.models.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("mailer_integration", "0011_invalidperson_newperson_synchronizationresult"),
    ]

    operations = [
        migrations.CreateModel(
            name="SynchronizationPerson",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("email", models.CharField(max_length=100)),
                (
                    "first_name",
                    models.CharField(
                        blank=True, max_length=255, verbose_name="First Name"
                    ),
                ),
                (
                    "last_name",
                    models.CharField(
                        blank=True, max_length=255, verbose_name="Last Name"
                    ),
                ),
                (
                    "successfully_added",
                    models.BooleanField(
                        verbose_name=django.db.models.fields.BooleanField
                    ),
                ),
                (
                    "sync_result",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="persons",
                        to="mailer_integration.synchronizationresult",
                    ),
                ),
            ],
        ),
        migrations.RemoveField(
            model_name="newperson",
            name="sync_result",
        ),
        migrations.DeleteModel(
            name="InvalidPerson",
        ),
        migrations.DeleteModel(
            name="NewPerson",
        ),
    ]



# Generated by Django 3.2.5 on 2021-09-28 07:45

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("core", "0012_auto_20210928_1158"),
    ]

    operations = [
        migrations.RenameModel(
            old_name="Category",
            new_name="Vocabulary",
        ),
        migrations.AlterModelOptions(
            name="vocabulary",
            options={
                "ordering": ["name"],
                "verbose_name": "Vocabulary",
                "verbose_name_plural": "Vocabularies",
            },
        ),
    ]



from django.conf import settings
from djmoney.settings import CURRENCY_CHOICES

DEFAULTS = {"PREFERRED_CURRENCIES": [i for i in CURRENCY_CHOICES if i[0] == "USD"]}

preferred_codes = getattr(settings, "BASXCONNECT", {}).get(
    "PREFERRED_CURRENCIES", DEFAULTS["PREFERRED_CURRENCIES"]
)
PREFERRED_CURRENCIES = [i for i in CURRENCY_CHOICES if i[0] in preferred_codes]
PREFERRED_CURRENCIES.sort(key=lambda e: preferred_codes.index(e[0]))



from bread import layout
from bread.utils import Link, ModelHref, pretty_modelname
from django.shortcuts import get_object_or_404
from django.utils.translation import gettext_lazy as _

from basxconnect.contributions.models import Contribution
from basxconnect.core.layouts.editperson.common import utils
from basxconnect.core.models import Person

R = layout.grid.Row
C = layout.grid.Col


def contributions_tab(request):
    person = get_object_or_404(Person, pk=request.resolver_match.kwargs["pk"])
    return layout.tabs.Tab(
        _("Contributions"),
        utils.grid_inside_tab(
            R(
                utils.tiling_col(
                    layout.datatable.DataTable.from_queryset(
                        person.contributions.all(),
                        columns=[
                            "_import.date",
                            "date",
                            "note",
                            "debitaccount",
                            "creditaccount",
                            "amount_formatted",
                        ],
                        title="",
                        primary_button=layout.button.Button.fromlink(
                            Link(
                                href=ModelHref(
                                    Contribution, "add", query={"person": person.id}
                                ),
                                label=_("Add %s") % pretty_modelname(Contribution),
                            ),
                            icon=layout.icon.Icon("add", size=20),
                        ),
                        backurl=request.get_full_path(),
                        prevent_automatic_sortingnames=True,
                    )
                )
            )
        ),
    )



# Generated by Django 3.2.4 on 2021-08-24 12:36

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("mailer_integration", "0006_alter_mailingpreferences_status"),
    ]

    operations = [
        migrations.AlterField(
            model_name="mailingpreferences",
            name="interests",
            field=models.ManyToManyField(blank=True, to="mailer_integration.Interest"),
        ),
    ]



import logging

import bread
import django
import htmlgenerator as hg
from bread import layout, menu
from bread.layout.components.button import Button
from bread.layout.components.forms import Form
from bread.utils import Link, reverse_model
from bread.views import EditView, ReadView, layoutasreadonly
from django.apps import apps
from django.conf import settings
from django.forms import forms
from django.http import HttpResponse, HttpResponseRedirect
from django.shortcuts import get_object_or_404
from django.utils.module_loading import import_string
from django.utils.translation import gettext_lazy as _
from django.views.decorators.csrf import csrf_exempt

from ... import models
from ...layouts.editperson import legalperson, naturalperson, personassociation
from ...layouts.editperson.common.head import editperson_head
from ...layouts.editperson.common.relationships_tab import relationshipstab

R = layout.grid.Row
C = layout.grid.Col


class NaturalPersonEditView(EditView):
    def get_layout(self):
        return personform_shortcut(
            self.request,
            editnaturalperson_form(self.request),
        )


class NaturalPersonReadView(ReadView):
    def get_layout(self):
        return personform_shortcut(
            self.request,
            editnaturalperson_form(self.request),
        )


class LegalPersonEditView(EditView):
    def get_layout(self):
        return personform_shortcut(
            self.request,
            editlegalperson_form(self.request),
        )


class LegalPersonReadView(ReadView):
    def get_layout(self):
        return layoutasreadonly(
            personform_shortcut(
                self.request,
                editlegalperson_form(self.request),
            )
        )


class PersonAssociationEditView(EditView):
    def get_layout(self):
        return personform_shortcut(
            self.request,
            editpersonassociation_form(self.request),
        )


class PersonAssociationReadView(ReadView):
    def get_layout(self):
        return personform_shortcut(
            self.request,
            editpersonassociation_form(self.request),
        )


def personform_shortcut(request, formlayout):
    return hg.BaseElement(
        layout.grid.Grid(
            editperson_head(request),
            formlayout,
            gutter=False,
        )
    )


def editnaturalperson_form(request):
    return editperson_form(
        request,
        naturalperson.base_data_tab,
        naturalperson.mailings_tab,
    )


def editpersonassociation_form(request):
    return editperson_form(
        request,
        personassociation.base_data_tab,
        personassociation.mailings_tab,
    )


def editlegalperson_form(request):
    return editperson_form(
        request,
        legalperson.base_data_tab,
        legalperson.mailings_tab,
    )


def editperson_form(request, base_data_tab, mailings_tab):
    return R(
        C(
            layout.grid.Grid(
                layout.tabs.Tabs(
                    *editperson_tabs(base_data_tab, mailings_tab, request),
                    tabpanel_attributes={
                        "_class": "tile-container",
                        "style": "padding: 0;",
                    },
                    labelcontainer_attributes={
                        "_class": "tabs-lg",
                        "style": "background-color: white;",
                    },
                ),
                gutter=False,
            ),
        ),
    )


def editperson_tabs(base_data_tab, mailing_tab, request):

    from django.apps import apps

    if apps.is_installed("basxconnect.contributions"):
        from ...layouts.editperson.common import contributions_tab

        return [
            base_data_tab(request),
            relationshipstab(request),
            mailing_tab(request),
            contributions_tab.contributions_tab(request),
        ]
    return [
        base_data_tab(request),
        relationshipstab(request),
        mailing_tab(request),
    ]


@csrf_exempt
def togglepersonstatus(request, pk: int):
    if request.method == "POST":
        person = get_object_or_404(models.Person, pk=pk)
        person.active = not person.active
        person.save()
    return HttpResponse(
        _("%s is %s") % (person, _("Active") if person.active else _("Inactive"))
    )


def confirm_delete_email(request, pk: int):
    email = models.Email.objects.get(id=pk)
    enable_delete_mailer_contact_checkbox = apps.is_installed(
        "basxconnect.mailer_integration"
    ) and hasattr(email, "subscription")

    fields = []
    if enable_delete_mailer_contact_checkbox:

        from basxconnect.mailer_integration.settings import MAILER

        class DeleteMailerSubscriptionForm(forms.Form):
            delete_mailer_contact = django.forms.BooleanField(
                label=_("Delete linked %s subscription as well") % MAILER.name(),
                required=False,
            )

        fields.append("delete_mailer_contact")

        if request.method == "POST":
            form = DeleteMailerSubscriptionForm(request.POST)
            if form.is_valid():
                person = email.person
                if enable_delete_mailer_contact_checkbox and form.cleaned_data.get(
                    "delete_mailer_contact"
                ):
                    try:
                        from basxconnect.mailer_integration.settings import MAILER

                        MAILER.delete_person(email.email)
                    except Exception:
                        logging.error("tried to delete person from mailer but failed")

                email.delete()
                person.refresh_from_db()
                person.save()
                return HttpResponseRedirect(
                    reverse_model(person, "read", kwargs={"pk": person.pk})
                )
        else:
            form = DeleteMailerSubscriptionForm()
    else:
        if request.method == "POST":
            form = forms.Form(request.POST)
            if form.is_valid():
                email.delete()
                return HttpResponseRedirect(
                    reverse_model(email.person, "read", kwargs={"pk": email.person.pk})
                )
        else:
            form = forms.Form()

    return layout.render(
        request,
        import_string(settings.DEFAULT_PAGE_LAYOUT)(
            menu.main,
            Form(
                form,
                hg.BaseElement(
                    hg.H3(_("Delete email %s") % email.email),
                    *(bread.layout.forms.FormField(field) for field in fields),
                ),
                hg.DIV(
                    Button.fromlink(
                        Link(
                            href=reverse_model(
                                email.person, "read", kwargs={"pk": email.person.pk}
                            ),
                            label=_("Cancel"),
                            iconname=None,
                        ),
                        buttontype="tertiary",
                    ),
                    hg.DIV(style="width: 1rem"),
                    *layout.forms.helpers.Submit(_("Confirm")),
                    style="display: flex; ",
                ),
            ),
        ),
    )



import htmlgenerator as hg
from bread.layout import button
from django.db import models
from django.utils.translation import gettext_lazy as _
from django_countries.fields import CountryField
from phonenumber_field.modelfields import PhoneNumberField

from .persons import Person
from .utils import Term


class Address(models.Model):
    person = models.ForeignKey(
        Person,
        on_delete=models.CASCADE,
        related_name="%(app_label)s_%(class)s_list",
    )
    person.verbose_name = _("Person")

    @classmethod
    def get_contact_related_fieldnames(cls):
        for subclass in cls.__subclasses__():
            yield f"{subclass._meta.app_label}_{subclass._meta.model_name}_list"

    class Meta:
        abstract = True


class Email(Address):
    email = models.EmailField(_("Email"))
    type = models.ForeignKey(
        Term,
        on_delete=models.SET_NULL,
        null=True,
        blank=True,
        related_name="type_%(app_label)s_%(class)s_list",
        limit_choices_to={"vocabulary__slug": "emailtype"},
    )
    type.verbose_name = _("Type")

    def asbutton(self):
        return hg.DIV(
            hg.SPAN(self.email, style="margin-right: 0.25rem"),
            hg.SPAN(style="flex-grow: 1"),
            button.Button(
                icon="email",
                onclick=f"window.location = 'mailto:{self.email}';",
                buttontype="ghost",
                _class="bx--overflow-menu",
            ),
            style="display: flex; flex-wrap: nowrap; align-items: center",
        )

    asbutton.verbose_name = _("Email")

    def __str__(self):
        return str(self.email)

    class Meta:
        verbose_name = _("Email address")
        verbose_name_plural = _("Email addresses")


class Web(Address):
    url = models.URLField(_("Url"))
    type = models.ForeignKey(
        Term,
        on_delete=models.SET_NULL,
        null=True,
        blank=True,
        related_name="type_%(app_label)s_%(class)s_list",
        limit_choices_to={"vocabulary__slug": "urltype"},
    )
    type.verbose_name = _("Type")

    def __str__(self):
        return str(self.url)

    class Meta:
        verbose_name = _("Web address")
        verbose_name_plural = _("Web addresses")


class Phone(Address):
    number = PhoneNumberField(_("Number"))
    type = models.ForeignKey(
        Term,
        on_delete=models.SET_NULL,
        null=True,
        blank=True,
        limit_choices_to={"vocabulary__slug": "phonetype"},
    )
    type.verbose_name = _("Type")

    def __str__(self):
        if self.type:
            return f"{self.number} ({self.type})"
        return str(self.number)

    class Meta:
        verbose_name = _("Phone number")
        verbose_name_plural = _("Phone numbers")


class Fax(Address):
    number = PhoneNumberField(_("Number"))
    type = models.ForeignKey(
        Term,
        on_delete=models.SET_NULL,
        null=True,
        blank=True,
        limit_choices_to={"vocabulary__slug": "phonetype"},
    )
    type.verbose_name = _("Type")

    def __str__(self):
        if self.type:
            return f"{self.number} ({self.type})"
        return str(self.number)

    class Meta:
        verbose_name = _("Fax number")
        verbose_name_plural = _("Fax numbers")


class Postal(Address):
    country = CountryField(_("Country"))
    address = models.TextField(_("Address"), blank=True)
    postcode = models.CharField(_("Post Code"), max_length=16, blank=True)
    city = models.CharField(_("City"), max_length=255, blank=True)
    type = models.ForeignKey(
        Term,
        on_delete=models.SET_NULL,
        null=True,
        blank=True,
        related_name="type_%(app_label)s_%(class)s_list",
        limit_choices_to={"vocabulary__slug": "addresstype"},
    )
    type.verbose_name = _("Type")
    valid_from = models.DateField(_("Valid from"), blank=True, null=True)
    valid_until = models.DateField(_("Valid until"), blank=True, null=True)

    def __str__(self):
        ret = [self.address]
        if self.postcode:
            ret.append(f"{self.postcode} {self.city}")
        else:
            ret.append(self.city)
        ret.append(self.country.name)
        return ", ".join(ret)

    class Meta:
        verbose_name = _("Postal address")
        verbose_name_plural = _("Postal addresses")
